"""Entrypoint de développement qui parle au code local."""

from scripts.dev_main_launcher import _add_src_to_path

_add_src_to_path()

import pymox_kit._globals as g

g.main()

from pymox_kit import *
import sys
import scripts.various as kit_dev

# from pymox_kit._globals import main as globals_main
# globals_main()

# from scripts.compare_runner import run_package_comparer


def main() -> int | None:

    cls()
    print(f"\n\n\t\t\t\t{SB}MON LOCAL MAIN MODE{R}\n")
    print(f"Bon {GREEN}{SI}{SB}code{R} !\n")

    # Pour comparer local (./main) et réel (./r_main)
    # return run_package_comparer(package, label=label)
    g.main()

    end()
    # end('Local Kit Dev')


if __name__ == "__main__":
    raise SystemExit(main())



# convert_readme.py

# with open("README.md", "r", encoding="cp1252", errors="replace") as f:
#     content = f.read()
# with open("README2.md", "w", encoding="utf-8") as f:
#     f.write(content)

with open("README.md", "r", encoding="utf-8", errors="replace") as f:
    content = f.read()
print(content)



"""Point d'entrée qui expose un `main()` plaçant le code local dans `sys.path`."""

from __future__ import annotations

import sys
from pathlib import Path
from typing import Sequence


def _add_src_to_path() -> None:
    root = Path(__file__).resolve().parent.parent
    src = root / "src"
    if not src.exists():
        return
    entry = str(src)
    if entry not in sys.path:
        sys.path.insert(0, entry)


def main(argv: Sequence[str] | None = None) -> int:
    """Appelle `pymox_kit.main` avec `src` dans `sys.path`."""
    _add_src_to_path()
    from pymox_kit.main import main as kit_main

    return kit_main(argv)



"""Entrypoint réel qui invoque `pymox_kit` comme un utilisateur."""

from __future__ import annotations

import sys
from pathlib import Path
from pymox_kit import *
# import scripts.common_footer as kit_dev

# from scripts.compare_runner import run_package_comparer


def _prefer_installed_package() -> None:
    repo_root = Path(__file__).resolve().parent
    local_src = str((repo_root / "src").resolve())
    sys.path[:] = [
        path
        for path in sys.path
        if Path(path).resolve().as_posix() != Path(local_src).resolve().as_posix()
    ]


def main() -> int | None:
    _prefer_installed_package()
    cls()

    print(f"\n\n\t\t\t\t{SB}MON REAL USER MODE{R}\n")

    # Pour comparer local (./main) et réel (./r_main)
    # return run_package_comparer(pk, label=label)
    
    end("Real Kit Use")

if __name__ == "__main__":
    raise SystemExit(main())



import locale, os, shutil, sys
# from flask import cli

locale.setlocale(locale.LC_ALL, "fr_FR")

# CLIW = SIMU_CLIW if SIMU_CLIW else CLIWR
# LG = "\n" + "-" * CLIWR
# from ._cli_utils import *

# print ((str(CLIW)+' ')*55)

##########################################################################
# SIMU_CliW = 40  # @i Si on veut Pour simuler une cliW sinon: Commenter #
SLEEP_DURATION = 0.7  # @i Tempo des affichages en secondes des parties  #
IDEAL_CLIWS = range(50, 61)  # Utiliser 55 col. est conseillé            #
##########################################################################


def ansi(n):
    return f"\x1b[0;9{n}m"


def ansi_style(n):
    return f"\x1b[{n}m"


# x = 0 : noir - 31 : rouge - 32 : vert - 33 : jaune - 34 : bleu - 35 : magenta - 36 : cyan - 37 : blanc
# 3x pour encre, 4x pour fond, 7 reverse, 10x fonds vifs
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = [ansi(i) for i in range(8)]
ST = "\x1b[30;43m"  # Stabilo effect
# Reset, Style Dim(Pâle), Bold, Italic, Underline, Reverse, Strikethrough
R, SB, SD, SI, SU, SR, SS = [ansi_style(i) for i in [*range(5), 7, 9]]


def main():
    print(
        f"{GREEN}{SB}Green{R} {RED}{SD}Red{R} {YELLOW}{SI}Yellow{R} {BLUE}{SU}Blue{R} {MAGENTA}{SR}Magenta{R} {CYAN}{SS}Cyan{R} {WHITE}White{R}"
    )


if __name__ == "__main__":
    raise SystemExit(main())

__all__ = [
    "BLACK",  # Noir"
    "RED",  # Rouge
    "GREEN",  # Vert
    "YELLOW",  # Jaune
    "BLUE",  # Bleu
    "MAGENTA",
    "CYAN",
    "WHITE",  # Blanc
    "SB",  # Style Bold (gras)
    "SD",  # Style Dim (pâle)
    "SI",  # Style Italic (italique)
    "SU",  # Style Underline (souligné)
    "SR",  # Style Reverse (inverse)
    "SS",  # Style Strikethrough (barré)
    "R",  # Reset (réinitialiser)
]



"""Runner commun pour comparer exécution locale vs package installé."""

from __future__ import annotations

import os
from pathlib import Path
from types import ModuleType

import dotenv


def run_package_comparer(package: ModuleType, *, label: str) -> int:
    """Exécute une démo standard pour un package `pymox_kit` importé."""

    dotenv.load_dotenv()

    color_reset = getattr(package, "R", "")
    color_red = getattr(package, "RED", "")
    color_cyan = getattr(package, "CYAN", "")
    hello = getattr(package, "hello", None)
    # print(f"{color_red}→ {label}{color_reset}")
  
    if os.getenv("PYMOX_DEBUG_IMPORT") == "1":
        print(f"[DEBUG] pymox_kit from: {getattr(package, '__file__', 'unknown')}")

    print("-" * 55)
    
    if callable(hello):
        print(f"{color_cyan}YEAHHH{color_reset}")
        print("\n260304 :\n")
        print(hello())
        print("-" * 55)

    print("CWD =", Path.cwd())
    print("ENV FILE EXISTS =", (Path(__file__).resolve().parent.parent / ".env").exists())
    print("DEBUG =", os.getenv("PYMOX_DEBUG_IMPORT"))
    print()
    return 0



from pymox_kit import *


def all_box_cars():
    """Voir tous les caractère box - 0x2500 → ALT + 2500 => ─"""
    for i in range(0x2400, 0x2580):
        print(f"{i} (0x{i:04X}) → {chr(i) * (CLIW-20)}")



import json, os, sys, shutil, time
from ._globals import *
from ._cli_utils import *
from ._tools import *

# from diskcache import Cache ❌ À tester // cache 'maison'

CACHE_FILE = os.path.join(os.path.expanduser("~"), ".cli_width_cache.json")

def clear():
    # Try ANSI first (works in most modern terminals), then shell fallbacks.
    try:
        print("\033[2J\033[3J\033[H", end="", flush=True)
    except Exception:
        pass

    def _run_cmd(cmd):
        try:
            return subprocess.run(
                cmd,
                check=False,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            ).returncode == 0
        except Exception:
            return False

    term = os.environ.get("TERM", "")
    if term and shutil.which("tput") and _run_cmd(["tput", "clear"]):
        return

    if os.name == "nt":
        # Git Bash/mintty usually provides clear; cmd builtin is a Windows fallback.
        if shutil.which("clear") and _run_cmd(["clear"]):
            return
        _run_cmd(["cmd", "/c", "cls"])
        return

    if shutil.which("clear"):
        _run_cmd(["clear"])



def cls(title=None, filename="", page=None):
    """Réinitialise la console (CLI) ou la page (Flet).
    Affiche title sauf si title=0.
    """

    # cliWAnalysis() # //2ar
    
    # if title != 0: ❌ A remettre
    #     setTitle(title, filename)
    
    if page is None and hasattr(title, "clean") and callable(getattr(title, "clean")):
        page = title
        title = None

    if page is not None and hasattr(page, "clean") and callable(page.clean):
        page.clean()

        if title not in (None, 0) and hasattr(page, "title"):
            page.title = str(title)

        if hasattr(page, "update") and callable(page.update):
            page.update()
        return

    clear()

    print(f"{RED}{SR}Oki{R} ! {GREEN}{SI}Let's go...{R} !")

TTL = 7  # 7

def read_cache():
    if not os.path.exists(CACHE_FILE):
        return None
    try:
        with open(CACHE_FILE, "r", encoding="utf-8") as f:
            data = json.load(f)

        if time.time() - data["ts"] < TTL:
            return data["width"]
    except (OSError, json.JSONDecodeError, KeyError, TypeError, ValueError):
        # Cache invalide ou corrompu: on repart proprement au prochain calcul.
        try:
            os.remove(CACHE_FILE)
        except OSError:
            pass

    return None


def write_cache(width):
    with open(CACHE_FILE, "w", encoding="utf-8") as f:
        json.dump({"width": width, "ts": time.time()}, f)

def get_cli_width():
    w = read_cache()
    if w is not None:
        # print(f" {GREEN}Cached{R} -" * 4)
        return w
    w = get_cli_width_process()
    # str = f"{RED}Process...{R}"
    print(f"{str}", f"→ {RED}{SB}{w}{R} cols\n")
    write_cache(w)
    return w

def get_cli_width_process(default=80):
    """
    Récupère la largeur réelle de la console avec plusieurs stratégies.

    Ordre volontairement choisi (et testé en conditions réelles) :
    1. Variables d'environnement explicites (prioritaires)
    2. Windows : interrogation directe de la console via WinAPI
    3. os.get_terminal_size() sur plusieurs flux (stdout, stderr, stdin)
    4. shutil.get_terminal_size() avec fallback
    5. Valeur par défaut

    Cet ordre est important : dans certains environnements (stdout non TTY,
    stderr/stdin TTY, consoles intégrées, Flet, IDE...), shutil renvoie
    systématiquement la valeur de fallback (80), alors que la WinAPI ou
    certains flux donnent la vraie largeur.
    """

    # for suffix in ("out", "err", "in"):
    #     stream = getattr(sys, f"std{suffix}")
    #     name = f"sys.std{suffix}.isatty()"
    #     print(f"{name:<19} = {stream.isatty()}")

    # 1) Variables d'environnement
    for env_name in ("PY_CLI_WIDTH", "CLI_WIDTH", "COLUMNS"):
        env_val = os.environ.get(env_name)
        if env_val and env_val.isdigit() and int(env_val) > 0:
            return int(env_val)

    # 2) Windows : WinAPI sur stdout / stderr / stdin
    if os.name == "nt":
        try:
            import ctypes
            from ctypes import wintypes

            class COORD(ctypes.Structure):
                _fields_ = [("X", wintypes.SHORT), ("Y", wintypes.SHORT)]

            class SMALL_RECT(ctypes.Structure):
                _fields_ = [
                    ("Left", wintypes.SHORT),
                    ("Top", wintypes.SHORT),
                    ("Right", wintypes.SHORT),
                    ("Bottom", wintypes.SHORT),
                ]

            class CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
                _fields_ = [
                    ("dwSize", COORD),
                    ("dwCursorPosition", COORD),
                    ("wAttributes", wintypes.WORD),
                    ("srWindow", SMALL_RECT),
                    ("dwMaximumWindowSize", COORD),
                ]

            get_std_handle = ctypes.windll.kernel32.GetStdHandle
            get_csbi = ctypes.windll.kernel32.GetConsoleScreenBufferInfo
            get_std_handle.argtypes = [wintypes.DWORD]
            get_std_handle.restype = wintypes.HANDLE
            get_csbi.argtypes = [
                wintypes.HANDLE,
                ctypes.POINTER(CONSOLE_SCREEN_BUFFER_INFO),
            ]
            get_csbi.restype = wintypes.BOOL

            for std_handle in (-11, -12, -10):  # stdout, stderr, stdin
                handle = get_std_handle(std_handle)
                if not handle or handle == wintypes.HANDLE(-1).value:
                    continue
                csbi = CONSOLE_SCREEN_BUFFER_INFO()
                if get_csbi(handle, ctypes.byref(csbi)):
                    cols = int(csbi.srWindow.Right - csbi.srWindow.Left + 1)
                    if cols > 0:
                        return cols
        except Exception:
            pass

    # 3) os.get_terminal_size() sur plusieurs flux
    for stream in (
        getattr(sys, "__stdout__", None),
        sys.stdout,
        getattr(sys, "__stderr__", None),
        sys.stderr,
        getattr(sys, "__stdin__", None),
        sys.stdin,
    ):
        try:
            if stream is None:
                continue
            cols = os.get_terminal_size(stream.fileno()).columns
            if cols > 0:
                return cols
        except Exception:
            continue

    # 4) shutil.get_terminal_size() avec fallback
    try:
        cols = shutil.get_terminal_size(fallback=(default, 24)).columns
        if cols > 0:
            return cols
    except Exception:
        pass

    # 5) Fallback final
    return default


# print("ID get_cli_width =", id(get_cli_width))
# print("ID get_cli_width_process =", id(get_cli_width_process))

CLIW = get_cli_width()


def end(mode="Dev"):
    """
    Args:
        mode (str, optional): _description_. Defaults to "Dev".
    """
    label = mode
    biptime = bip_time()
    sp_w = CLIW - len(label) - len(biptime) - 2  # -2 pour les 2 , => 2 espaces
    w =CLIW
    # print(f"{w = }")
    
    # ❌ donne pypi version & cliw
    print()
    print(f"{chr(0x2500)}" * CLIW)
    print(f"{CYAN}{SB}{label}{R}", " " * sp_w, biptime)

__all__ = ["cls", "CLIW", "end"]



import locale, inspect, os, sys, time, winsound

from ._globals import *
from ._cli_utils import *

CLIW = CLIWR = 50

# ❌ Expo inutile → sera la der ligne du exit() ------ 24:00:59 ---
def bip_time() -> str:
    """Affiche un marqueur temporel dans la console."""
    # print("\a")
    # winsound.PlaySound("bip.wav", winsound.SND_FILENAME)
    # winsound.MessageBeep()

    winsound.Beep(2500, 30)
    # time.sleep(0.05)
    winsound.Beep(2500, 30)
    return time.strftime("%H:%M:%S")


def is_page(obj):
    return hasattr(obj, "clean") and callable(obj.clean)


def sl(
    color: str | None = None,
    w: int = CLIW,
    trait="─",
    finTrait="",
    toPrint: bool = True,
) -> str | None:
    """Simple Line\nparm: BLUE, RED, ... or FRENCH"""
    global lineColor

    if color == "french":
        lineCode = frenchLine(trait=trait)
    else:
        lineColor = color if color else GREEN if CLIWR in IDEAL_CLIWS else RED
        lineCode = (
            f"\033[0;3{lineColor};40m" + f"{trait}{finTrait}" * w + "\033[0;37;40m"
        )

    if toPrint:
        print(lineCode)
        return None
    else:
        return lineCode


def frenchLine(
    w: int | None = CLIWR,
    trait="─",
) -> str:
    """w is None | w != CLIWR Print a BLUE-WHITE-RED line"""

    def partsLength(totalLength: int) -> tuple:
        """Calculate pure length of each part (tuple) first and third (egal), central part"""
        a = totalLength // 3
        if totalLength % 3 == 0:  # Cas où n = 1, 4, 7,...
            b = a
        elif totalLength % 3 == 1:  # Cas où n = 2, 5, 8,...
            b = a + 1
        else:  # Cas où n = 3, 6, 9,...
            a += 1
            b = a - 1
        return (a, b)

    # w = 21
    pL = partsLength(w)  # (a, b) patriotPartLength
    # print("w =", w, "→", *pL, pL[0])
    # print("-" * 65 + "8888")

    colorsCodes = [4, 7, 1, 7]

    endsLine = f"{trait}" * pL[0]
    centerLine = f"{trait}" * pL[1]

    endColors = ""
    (partBlue, partWhite, partRed, endColors) = (
        "\033[1;34m" + endsLine,
        "\033[1;37m" + centerLine,
        "\033[1;31m" + endsLine,
        "\033[0;37m" + endColors,
    )
    sFinale = partBlue + partWhite + partRed + endColors

    # print(partBlue, len(partBLUE), "(partBLUE)")
    # print(partWHITE, len(partpartWhiteWHITE), "(partWHITE)")
    # print(partRed, len(partRed), "(partRed)")
    # print(endColors, len(endColors), "(endColors)")
    # print(
    #     sFinale,
    #     len(partBlue + partWhite + partRed + endColors),
    #     "(partBlue + partWhite + partRed + endColors)",
    # )

    # print(len(sFinale), "(len(sFinale))")
    # print(rawStrLength(sFinale), "(rawStrLength(sFinale))")

    # print("\n" + "-" * 21, "21", "(Réfce.)")
    # name = " Lionel "

    # complete = sFinale + name

    # print(
    #     sFinale + name,
    #     rawStrLength(sFinale + name),
    #     "(sFinale + name)",
    # )
    return sFinale
    exit()  # 2ar


def nf(f, dec=2):
    "Number Format 123456.789 → 123 456,79"
    try:
        f = float(f)
        return locale.format_string(f"%.{dec}f", f, grouping=True)
    except ValueError:
        src = caller_info()
        # print(src)
        print(
            f"⚠️ Errorfor nf() in main_tools:\n\033[1;31mBad data type ({type(f).__name__}) -> {f} (Line {src[2]} in {src[0]}){EB}"
        )
        return str(f)


def caller_info(justfilename: bool = False, level=2) -> tuple | str:
    """
    Return (tuple) Path of caller file, caller function name, index of line where is the instruction.\nIf argument is True (or 1): (str) Just theCcller file name
    """

    frame = inspect.currentframe()

    # Vérifier la profondeur de la pile avant d'accéder à f_back plusieurs fois
    for _ in range(level):
        if frame is not None and frame.f_back is not None:
            frame = frame.f_back
        else:
            return "Frame introuvable"

    # Vérifier si frame est toujours valide avant de l'utiliser
    if frame is None:
        return "Frame introuvable"

    try:
        callerFilePath = os.path.relpath(inspect.getfile(frame))  # Chemin relatif
    except TypeError:
        return "Impossible de récupérer le fichier appelant"
    # Obtenir le numéro de ligne
    # callerLineNumber = frame.f_lineno
    callerLineNumber = int(frame.f_lineno) if frame is not None else -1

    # Nom de la fonction appelante
    function_name = frame.f_code.co_name
    context = "main" if function_name == "<module>" else f"{function_name}()"

    if justfilename:
        # return callerFilePath # 2ar vérif si dessous ok
        return os.path.basename(callerFilePath)
    return callerFilePath, context, callerLineNumber


def get_caller_function() -> str | None:
    """Return caller function if exists"""
    stack = inspect.stack()
    if len(stack) > 2:  # Vérifie qu'il y a une fonction appelante
        caller_frame = stack[2]
        caller_function = caller_frame.function  # Récupère le nom de la fonction
        return caller_function
    return None


__all__ = ["sl", "nf", "bip_time"]



"""Entrypoint package : `python -m pymox_kit`.

Offre un petit CLI :
- `--version` / `-V` : affiche la version (installation locale ou PyPI)
- sans option : délègue à `pymox_kit.main.main()` si présent, sinon appelle
  `pymox_kit.kit.hello()` / `bye()`.
"""

from __future__ import annotations

from typing import Sequence, Optional


def _get_package_version() -> str:
    try:
        from importlib.metadata import version, PackageNotFoundError

        try:
            # distribution name may be either 'pymox_kit' or 'pymox-kit'
            return version("pymox_kit")
        except PackageNotFoundError:
            try:
                return version("pymox-kit")
            except PackageNotFoundError:
                pass
    except Exception:
        pass

    # fallback to package attribute
    try:
        import pymox_kit

        return getattr(pymox_kit, "__version__", "unknown")
    except Exception:
        return "unknown"


def main(argv: Optional[Sequence[str]] = None) -> int:
    import argparse

    parser = argparse.ArgumentParser(
        prog="python -m pymox_kit", description="PyMoX Kit CLI"
    )
    parser.add_argument(
        "-V", "--version", action="store_true", help="Afficher la version"
    )
    args = parser.parse_args(argv)

    if args.version:
        print(_get_package_version())
        return 0

    # Délégation principale vers pymox_kit.main.main() si disponible
    try:
        from importlib import import_module

        mod = import_module("pymox_kit.main")
        if hasattr(mod, "main"):
            mod.main()
            return 0
    except Exception:
        pass

    # Sinon usage direct des helpers
    try:
        from .kit import hello, bye

        print(hello())
        print(bye())
        return 0
    except Exception as exc:
        print("Erreur lors de l'exécution du package:", exc)
        return 2


if __name__ == "__main__":
    raise SystemExit(main())
